<?php
/* References
 * https://github.com/chipbennett/oenology/blob/master/functions/options.php#L84
 * http://www.chipbennett.net/2011/02/17/incorporating-the-settings-api-in-wordpress-themes/7/
 */
/* Add admin menu */
add_action( 'admin_menu', 'pm_hlshr_admin_menu' );
function pm_hlshr_admin_menu() {
	global $pm_hlshr_settings;
	
	// Create options page
	$pm_hlshr_settings = add_options_page(PM_HLSHR_PLUGIN_NAME.' Settings', PM_HLSHR_PLUGIN_NAME, 'manage_options', 'pm_hlshr_settings', 'pm_hlshr_options_page');
  	
  	// Load admin scripts when above page is called
  	add_action( 'load-'.$pm_hlshr_settings, 'pm_hlshr_adminscripts' );
}

// Enqueue admin scripts
function pm_hlshr_adminscripts() {
	global $pm_hlshr_settings;
	$screen = get_current_screen();

	// Enqueue scripts and styles
	wp_enqueue_script( 'pm-hlshr-admin', PM_HLSHR_DIR.'modules/js/admin.js', array('wp-color-picker', 'jquery' ), false, true );
	wp_enqueue_style( 'wp-color-picker' );
	wp_enqueue_style( 'pm-hlshr-admin', PM_HLSHR_DIR.'modules/css/admin.css', '', PM_HLSHR_VERSION, 'all' );
}

// WP Pointer Admin Tooltip - Make sure you use cookie to dismiss
add_action('admin_enqueue_scripts', 'pm_hlshr_enqueue_admin_scripts');
function pm_hlshr_enqueue_admin_scripts() {
    $dismissed = explode (',', get_user_meta (wp_get_current_user ()->ID, 'dismissed_wp_pointers', true));
    $not_in_array = !in_array ('pm_hlshr_settings_dismiss', $dismissed);

    if( $not_in_array ) {
	    wp_enqueue_style('wp-pointer');
	    wp_enqueue_script('wp-pointer');
	    //hook the pointer 
	    add_action('admin_print_footer_scripts', 'pm_hlshr_print_footer_scripts' );
	}
}
function pm_hlshr_print_footer_scripts() {
    $pointer_content = '<h3>'.PM_HLSHR_PLUGIN_NAME.'</h3>';
    $pointer_content .= '<p>Configure your settings here.</p>';
?>
   <script type="text/javascript">
   //<![CDATA[
   jQuery(document).ready( function($) {
    //jQuery selector to point to 
    $('#menu-settings').pointer({
        content: '<?php echo $pointer_content; ?>',
        position: 'top',
        close: function() {
            // This function is fired when you click the close button
            $.post (ajaxurl, {
		        pointer: 'pm_hlshr_settings_dismiss',
		        action: 'dismiss-wp-pointer'
		    });
        }
      }).pointer('open');
   });
   //]]>
   </script>
<?php
}

// Output the settings page
function pm_hlshr_options_page() {
	$tab = ( isset( $_GET['tab'] )  ? $_GET['tab'] : 'defaults' );
	$settings_section = 'pm_hlshr_settings_'.$tab.'_tab';
	?>
	<div class="wrap">
		<h2><?php echo PM_HLSHR_PLUGIN_NAME; ?> Settings</h2>
		<?php settings_errors( 'pm_hlshr_options' ); ?>
		<?php pm_hlshr_settings_page_tabs(); ?>

		<form action='options.php' method='post'>
			<?php 
			// Call functions to build form
			settings_fields( 'pm_hlshr_options' );
			//do_settings_sections( 'pm_hlshr_settings' );
			do_settings_sections( $settings_section );

			// Update settings based on tab			
			?>
			<p class="submit">
			<?php
			submit_button( 'Save Settings', 'primary', 'pm_hlshr_options[submit-'.$tab.']', false, array( 'id' => 'submit-'.$tab ) );
			submit_button( 'Reset', 'secondary', 'pm_hlshr_option[reset-'.$tab.']', false, array( 'id' => 'reset-'.$tab ) );
			?></p>
		</form>
	</div>
	<?php
}

// Create tabs
function pm_hlshr_get_settings_page_tabs() {
	$tabs = array(
		'defaults' => 'Defaults',
		'misc' => 'Miscellaneous'
		);
	return $tabs;
}

// HTML Markup for tabs
function pm_hlshr_settings_page_tabs( $current = 'defaults' ) {
	// Determine which tab to display
	if( isset( $_GET['tab'] ) ) :
		$current = $_GET['tab'];
	else:
		$current = 'defaults';
	endif;

	// Get list of tabs
	$tabs = pm_hlshr_get_settings_page_tabs();
	$links = array();

	// Create links for each tab
	foreach ($tabs as $tab => $name) :
		if( $tab == $current ) :
			$links[] = '<a class="nav-tab nav-tab-active" href="?page=pm_hlshr_settings&tab='.$tab.'">'.$name.'</a>';
		else :
			$links[] = '<a class="nav-tab" href="?page=pm_hlshr_settings&tab='.$tab.'">'.$name.'</a>';
		endif;
	endforeach;

	echo '<h2 class="nav-tab-wrapper">';
	foreach ( $links as $link ) {
		echo $link;
	}
	echo '</h2>';
}

// Register settings
add_action( 'admin_init', 'pm_hlshr_initialize_settings' );
function pm_hlshr_initialize_settings() {
	// Register fields
	register_setting( 
		'pm_hlshr_options',
		'pm_hlshr_options', 
		'pm_hlshr_validate_options' );

	// Add settings sections
	pm_hlshr_add_settings_sections();
	//add_settings_section( 'one', 'One Section', 'pm_hlshr_one_section_callback', 'pm_hlshr_settings' );

	// Add settings fields
	pm_hlshr_add_settings_fields();
	//add_settings_field( 'textfield', 'Text Field', 'pm_hlshr_textfield_field_callback', 'pm_hlshr_settings', 'one' );
}

/*---------------
 Sections
 ---------------*/
// Register multiple settings (called in admin_init)
function pm_hlshr_add_settings_sections() {
	// Array with settings identifiers
	$sects = array(
		'hilite_options' => array(
			'id' => 'hilite_options',
			'title' => 'Hilite',
			'page' => 'pm_hlshr_settings_defaults_tab'
			),
		'qte_box_design' => array(
			'id' => 'qte_box_design',
			'title' => 'Featured Quote Box Style',
			'page' => 'pm_hlshr_settings_defaults_tab'
			),
		'social_settings' => array(
			'id' => 'social_settings',
			'title' => 'Social Network',
			'page' => 'pm_hlshr_settings_defaults_tab'
			),
		'data' => array(
			'id' => 'data',
			'title' => 'Data Handling',
			'page' => 'pm_hlshr_settings_misc_tab'
			),
		);

	// Loop through each
	foreach ($sects as $s) {
		add_settings_section( 
			$s['id'].'_section',
			$s['title'].' Options',
			'pm_hlshr_'.$s['id'].'_section_callback',
			$s['page']
			);
	}
}

/*----------------
Sections callback
-----------------*/
// Section callback
function pm_hlshr_data_section_callback() {
}
function pm_hlshr_hilite_options_section_callback() {}
function pm_hlshr_qte_box_design_section_callback() {
}
function pm_hlshr_social_settings_section_callback() {}

/*------------------
Fields
------------------*/
// Register multiple fields (called in admin_init)
function pm_hlshr_add_settings_fields () {
	// Array with field info
	$fields = array(
		'enable_hilite' => array(
			'id' => 'enable_hilite',
			'title' => 'Enable highlighting feature',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'hilite_options_section'
			),
		'hilite_post_types' => array(
			'id' => 'hilite_post_types',
			'title' => 'Enable highlighting for',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'hilite_options_section'
			),
		'border' => array(
			'id' => 'border',
			'title' => 'Border',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'border_width' => array(
			'id' => 'border_width',
			'title' => 'Border width',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'border_color' => array(
			'id' => 'border_color',
			'title' => 'Border color',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'bgcolor' => array(
			'id' => 'bgcolor',
			'title' => 'Background color',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'boxstyle' => array(
			'id' => 'boxstyle',
			'title' => 'Box style',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'boxwidth' => array(
			'id' => 'boxwidth',
			'title' => 'Box width',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'qte_box_design_section'
			),
		'via' => array(
			'id' => 'via',
			'title' => 'Twitter user name',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'social_settings_section'
			),
		'usefb' => array(
			'id' => 'usefb',
			'title' => 'Use Facebook?',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'social_settings_section'
			),
		'fbappid' => array(
			'id' => 'fbappid',
			'title' => 'Facebook app ID',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'social_settings_section'
			),
		'fbtitle' => array(
			'id' => 'fbtitle',
			'title' => 'Facebook share title',
			'page' => 'pm_hlshr_settings_defaults_tab',
			'section' => 'social_settings_section'
			),
		'optsdata' => array(
			'id' => 'optsdata',
			'title' => 'Settings Data',
			'page' => 'pm_hlshr_settings_misc_tab',
			'section' => 'data_section'
			),
		);

	// Loop through each in array
	foreach ($fields as $f) {
		add_settings_field( 
			$f['id'],
			$f['title'],
			'pm_hlshr_'.$f['id'].'_field_callback',
			$f['page'],
			$f['section']
			);
	}
}

/*--------------------
Fields callback
--------------------*/
function pm_hlshr_hilite_post_types_field_callback() {
	global $pm_hlshr_opts;
	$hilite_post_type = ( empty( $pm_hlshr_opts['hilite_post_types'] ) ) ? array() : $pm_hlshr_opts['hilite_post_types'];
	$post_types = get_post_types( array( 'public' => true ) );
	unset( $post_types['attachment'] );
	$post_types = array_values( $post_types );

	foreach( $post_types as $post_type ) {
		?>
		<label><input type="checkbox" name="pm_hlshr_options[hilite_post_types][]" value="<?php echo sanitize_text_field( $post_type ); ?>" <?php if( in_array( $post_type, $hilite_post_type ) ) { echo "checked"; } ?>> <?php echo sanitize_text_field( $post_type ); ?></label>&nbsp;
		<?php
	} ?>
	<p>
		<small>Check to enable highlight and share for these post types</small>
	</p>
	<?php
}
function pm_hlshr_enable_hilite_field_callback() {
	global $pm_hlshr_opts;
	$enable_hilite = ( empty( $pm_hlshr_opts['enable_hilite'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['enable_hilite'] );
	?>
	<label><input type="checkbox" name="pm_hlshr_options[enable_hilite]" value="1" <?php checked( $enable_hilite, 1, true ); ?>> Yes</label><br>
	<small>Turn highlight and share feature</small>
	<?php
}
function pm_hlshr_border_field_callback() {
	global $pm_hlshr_opts;
	$border = ( empty( $pm_hlshr_opts['border'] ) ) ? '' : $pm_hlshr_opts['border'];
	?>
	<select name="pm_hlshr_options[border]">
		<option value="none" <?php selected( $border, '' ); ?>>None</option>
		<option value="solid" <?php selected( $border, 'solid' ); ?>>Solid</option>
		<option value="dotted" <?php selected( $border, 'dotted' ); ?>>Dotted</option>
		<option value="dashed" <?php selected( $border, 'dashed' ); ?>>Dashed</option>
	</select>
	<?php
}
function pm_hlshr_border_width_field_callback() {
	global $pm_hlshr_opts;
	$border_width = ( empty( $pm_hlshr_opts['border_width'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['border_width'] );
	?>
	<input type="text" name="pm_hlshr_options[border_width]" value="<?php echo $border_width; ?>"> px
	<?php
}
function pm_hlshr_border_color_field_callback() {
	global $pm_hlshr_opts;
	$border_color = ( empty( $pm_hlshr_opts['border_color'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['border_color'] );
	?>
	<input type="text" name="pm_hlshr_options[border_color]" class="color-picker" value="<?php echo $border_color; ?>">
	<?php
}
function pm_hlshr_bgcolor_field_callback() {
	global $pm_hlshr_opts;
	$bgcolor = ( empty( $pm_hlshr_opts['bgcolor'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['bgcolor'] );
	?>
	<input type="text" name="pm_hlshr_options[bgcolor]" class="color-picker" value="<?php echo $bgcolor; ?>">
	<?php
}
function pm_hlshr_boxstyle_field_callback() {
	global $pm_hlshr_opts;
	$boxstyle = ( empty( $pm_hlshr_opts['boxstyle'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['boxstyle'] );
	?>
	<select name="pm_hlshr_options[boxstyle]">
		<option value="none" <?php selected( $boxstyle, '' ); ?>>None</option>
		<option value="shadow" <?php selected( $boxstyle, 'shadow' ); ?>>Shadow</option>
		<option value="rounded" <?php selected( $boxstyle, 'rounded' ); ?>>Rounded</option>
		<option value="inset" <?php selected( $boxstyle, 'inset' ); ?>>Inset Shadow</option>
	</select>
	<?php
}
function pm_hlshr_boxwidth_field_callback() {
	global $pm_hlshr_opts;
	$boxwidth = ( empty( $pm_hlshr_opts['boxwidth'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['boxwidth'] );
	?>
	<input type="text" name="pm_hlshr_options[boxwidth]" id="boxwidth" value="<?php echo $boxwidth; ?>">%<br>
			<small>Enter numbers only. Leave empty for 100%</small>
	<?php
}
function pm_hlshr_via_field_callback() {
	global $pm_hlshr_opts;
	$via = ( empty( $pm_hlshr_opts['via'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['via'] );
	?>
	<input type="text" name="pm_hlshr_options[via]" id="via" value="<?php echo $via; ?>">  (optional)<br><small>User name will be add to tweets. Leave blank to disable</small>
	<?php
}
function pm_hlshr_usefb_field_callback() {
	global $pm_hlshr_opts;
	$usefb = ( empty( $pm_hlshr_opts['usefb'] ) ) ? 0 : sanitize_text_field( $pm_hlshr_opts['usefb'] );
	?>
	<input type="checkbox" id="usefb" name="pm_hlshr_options[usefb]" value="1" <?php checked($usefb, 1, true); ?>> <label for="usefb">Yes</label>
	<?php
}
function pm_hlshr_fbappid_field_callback() {
	global $pm_hlshr_opts;
	$fbappid = ( empty( $pm_hlshr_opts['fbappid'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['fbappid'] );
	?>
	<input type="text" name="pm_hlshr_options[fbappid]" value="<?php echo $fbappid; ?>"> (required when using Facebook)<br>
			<small>Get this from your <a href="https://developers.facebook.com/apps" target="new">Facebook apps center</a></small>
	<?php
}
function pm_hlshr_fbtitle_field_callback() {
	global $pm_hlshr_opts;
	$fbtitle = ( empty( $pm_hlshr_opts['fbtitle'] ) ) ? '' : sanitize_text_field( $pm_hlshr_opts['fbtitle'] );
	?>
	<input type="text" name="pm_hlshr_options[fbtitle]" value="<?php echo $fbtitle; ?>"> (optional)<br>
	<small>If left empty, the default "Great point!" will be used</small>
	<?php
}
function pm_hlshr_optsdata_field_callback() {
	// Be sure the field names are formatted for array!
	// Also be sure name does not have ''
	global $pm_hlshr_opts;
	$del_opt_settings = ( empty( $pm_hlshr_opts['del_opt_settings'] ) ) ? '' : $pm_hlshr_opts['del_opt_settings'];
	?>
	<input type="checkbox" name="pm_hlshr_options[del_opt_settings]" value="1" id="del_opt_settings" <?php checked( $del_opt_settings, '1', true ); ?>> <label for="del_opt_settings">Yes. Delete all settings data when I uninstall</label><br>
	<small>Use with caution! When checked, all settings will be deleted on plugin uninstall. Disabling will not delete data only uninstall.</small>
	<?php
}

// Validate options
function pm_hlshr_validate_options( $input ) {
	$pm_hlshr_opts = get_option( 'pm_hlshr_options' );
	$valid_input = $pm_hlshr_opts;

	// Determine which tab is submitted
	$defaults = ( !empty( $input['submit-defaults'] ) ? true : false );
	$misc = ( !empty( $input['submit-misc'] ) ? true : false );

	if( $defaults ) {
		// Allowed select values
		$allowed_borders = array('none', 'solid', 'dotted', 'dashed');
		$allowed_boxstyles = array('none', 'shadow', 'rounded', 'inset');

		// validate only those settings
		if( isset( $input['hilite_post_types'] ) ) {
			foreach ($input['hilite_post_types'] as $post_type) {
				$hilite_post_types[] = sanitize_text_field( $post_type );
			}
		}
		$enable_hilite = ( isset( $input['enable_hilite'] ) ) ? '1' : '';
		$border = ( in_array( $input['border'], $allowed_borders ) ) ? $input['border'] : '' ;
		$border_width = ( isset( $input['border_width'] ) && ( ctype_digit( $input['border_width'] ) == 1 ) ) ? sanitize_text_field( $input['border_width'] ) : '' ; 
		$border_color = ( preg_match( '/^#[a-f0-9]{6}$/i', $input['border_color'] ) == '1' ) ? $input['border_color'] : '' ;
		$bgcolor = ( preg_match( '/^#[a-f0-9]{6}$/i', $input['bgcolor'] ) == '1' ) ? $input['bgcolor'] : '' ;
		$boxstyle = ( in_array( $input['boxstyle'], $allowed_boxstyles ) ) ? $input['boxstyle'] : '' ;
		$boxwidth = ( absint( $input['boxwidth'] ) );
		$via = ( ctype_alnum( $input['via'] ) == 1) ? $input['via'] : '' ;
		$usefb = ( isset( $input['usefb'] ) ) ? '1' : '';
		$fbappid = ( ctype_digit($input['fbappid'] ) == 1 ) ? $input['fbappid'] : '';
		$fbtitle = sanitize_text_field( $input['fbtitle'] );

		// Build array to save
		$valid_input = array(
			'hilite_post_types' => $hilite_post_types,
			'enable_hilite' => $enable_hilite,
			'border' => $border,
			'border_width' => $border_width,
			'border_color' => $border_color,
			'bgcolor' => $bgcolor,
			'boxstyle' => $boxstyle,
			'boxwidth' => $boxwidth,
			'via' => $via,
			'usefb' => $usefb,
			'fbappid' => $fbappid,
			'fbtitle' => $fbtitle,
			'del_opt_settings' => $pm_hlshr_opts['del_opt_settings']
			);
	} elseif ( $misc ) {
		// validate only those settings
		$del_opt_settings = ( isset( $input['del_opt_settings'] ) ) ? 1 : 0 ;

		// Build array to save
		$valid_input = array(
			'hilite_post_types' => $pm_hlshr_opts['hilite_post_types'],
			'enable_hilite' => $pm_hlshr_opts['enable_hilite'],
			'border' => $pm_hlshr_opts['border'],
			'border_width' => $pm_hlshr_opts['border_width'],
			'border_color' => $pm_hlshr_opts['border_color'],
			'bgcolor' => $pm_hlshr_opts['bgcolor'],
			'boxstyle' => $pm_hlshr_opts['boxstyle'],
			'boxwidth' => $pm_hlshr_opts['boxwidth'],
			'via' => $pm_hlshr_opts['via'],
			'usefb' => $pm_hlshr_opts['usefb'],
			'fbappid' => $pm_hlshr_opts['fbappid'],
			'fbtitle' => $pm_hlshr_opts['fbtitle'],
			'del_opt_settings' => $del_opt_settings
			);
	}
	return $valid_input;
}